

import java.awt.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.event.*;

public class Demo extends Canvas implements KeyListener {

  private int [] buffer1;
  private int [] buffer2;
  
  private int [] screen;

  private Image screenImage;
  private MemoryImageSource mis;

  private int w;
  private int h;
  private boolean bSkip;
  private Fader fader;
  
  public boolean bLongPlay;

  public Demo(int w, int h) {
    this.w = w;
    this.h = h;
    bLongPlay = false;
    buffer1 = new int[ w * h ];
    buffer2 = new int[ w * h ];
    screen = new int[ w * h ];
    fader = new Fader();

    mis=new MemoryImageSource(w,h,ColorModel.getRGBdefault(),screen,0,w);
    mis.setAnimated(true);
    mis.setFullBufferUpdates(true);
    screenImage = createImage(mis);

    addKeyListener(this);
  }
  public void paint(Graphics g) {
    g.drawImage(screenImage,0,0,this);   
  }
  
  public void update(Graphics g) {
    paint(g);
  }


  public void keyPressed(KeyEvent e) {
     if (e.getKeyCode() == e.VK_ESCAPE)
      System.exit(0);
     else
       bSkip = true;
  }
  public void keyReleased(KeyEvent e) {}
  public void keyTyped(KeyEvent e) {}  
 

  public void render() {}

  public void nextFrame() {
    mis.newPixels(0, 0, w, h);
  }

  


  private final void sleep(int n) {
    n -= tdiff;
    if (n > 0) {
      try { 
        Thread.sleep(n);
      }catch(Exception e) {}
    }
  }

  private long time;
  private void startTimer() {
    bSkip = false;
    time = System.currentTimeMillis();
    last = time;
  }
  
  int tdiff = 0;
  long last = 0;
  private boolean secTimer(int sec) {
    long t2 = System.currentTimeMillis();
    tdiff =(int)(t2-last);
    last = t2;
    return (((t2 - time) < sec*1000) || bLongPlay) 
      && !bSkip;
  }


  private void copyFromTo(int [] from, int [] to) {
    int n = to.length;
    for (int i = 0; i < n; i++)
      to[i] = from[i];
  }

  private void fadeout() {
    copyFromTo(screen, buffer1);
    fader.restart();
    while( fader.fadeout(buffer1, screen) ) {
      nextFrame();
      sleep(10);
    }
  }


  public void run() {
    
  
    
    //bumpmap
    {
      Noise noise = new Noise(w,h);
      noise.render(buffer1);
      noise = null;
      System.gc();

      Bump bump = new Bump(w, h, buffer2, buffer1);
      startTimer();
      while (secTimer(6)) {
        bump.render(screen);
        nextFrame();
        sleep(15);
      }
      fadeout();
      bump = null;
      System.gc();
    }
    //Flare
    {
      Flare flare = new Flare(w, h);
      startTimer();
      while (secTimer(10)) {
        flare.render(screen);
        nextFrame();
        sleep(10);
      }
      fadeout();
      flare = null;
      System.gc();
    }

    //NoiseFade
    {
      Noise noise = new Noise(w,h);
      noise.init(0x5EED);
      noise.render(buffer1);
      noise.render(buffer2);
      fader.restart();
      startTimer();
      while (secTimer(7)) {
        if ( fader.render( buffer1, buffer2, screen) ) {
          nextFrame(); 
          sleep(20);  
        }
        else {
          nextFrame();
          int [] temp = buffer1;
          buffer1 = buffer2;
          buffer2 = temp;
          noise.render(buffer2);
          fader.restart();                
        }
      }
      //fadeout();
      noise = null;
      System.gc();
    }
    
    {
      copyFromTo(screen, buffer1);
      LookingGlass lg = new LookingGlass(w,h);
      startTimer();
      while (secTimer(10)) {
        lg.render(buffer1, screen);
        nextFrame();
        sleep(20);
      }
      fadeout();
      lg = null;
      System.gc();
    }
    //Plasma
    {
      Plasma plasma = new Plasma(w, h);
      startTimer();
      while (secTimer(10)) {
        plasma.render(screen);
        nextFrame();
        sleep(15);
      }
      plasma = null;
      fadeout();
      System.gc();
    }
   
    //Cassini
    {
      Cassini cassini = new Cassini(w,h);
      startTimer();
      while(secTimer(10)) {
        cassini.render(screen);
        nextFrame();
      }
      fadeout();
      cassini = null;
      System.gc();
    }
    //Electro
    {
      Electro electro = new Electro(w,h, buffer1);
      startTimer();
      while(secTimer(10)) {
        electro.render(screen);
        nextFrame();
      }
      electro = null;
      fadeout();
      System.gc();
    }
    //Mountains
    {
      Mountains mount = new Mountains(w, h);
      startTimer();
      while(secTimer(15)) {
        mount.render(screen);
        nextFrame();
        sleep(20);
      }
      mount = null;
      fadeout();
      System.gc();
    } 
    //Tunnel
    {
      Noise noise = new Noise(w,h);
      noise.render(buffer2);
      noise = null;
      System.gc();
      Tunnel tunnel = new Tunnel(w,h,buffer1, buffer2);
      startTimer();
      while(secTimer(15)) {
        tunnel.render(screen);
        nextFrame();
        sleep(15);
      }
      tunnel = null;
      //fadeout();
      System.gc();
    }
    //Mato
    {
      Mato mato = new Mato(w, h, buffer2);
      copyFromTo(screen, buffer1);
      startTimer();
      while(secTimer(10)) {
        mato.render(buffer1, screen);
        nextFrame();
        sleep(20);
      }
      mato = null;
      fadeout();
      System.gc();
    }
    //Fractal
    {
      Fractal fractal = new Fractal(w, h);
      startTimer();
      while (secTimer(15) && fractal.render(screen)) {
        nextFrame();
      }
      fractal = null;
      fadeout();
      System.gc();
    }

    //Fire
    {
      Fire fire = new Fire(w,h, buffer1);
      startTimer();
      while(secTimer(10)) {
        fire.render(screen);
        nextFrame();
        sleep(15);
      }
      fadeout();
      fire = null;
      System.gc();
    }

    //Rotor
    {
      Rotor rotor = new Rotor(w,h);
      startTimer();
      while(secTimer(10)) {
        rotor.render(buffer1, screen);
        nextFrame();
        sleep(15);
      }
      fadeout();
      rotor = null;
      System.gc();
    }

    //Fireworks
    {
      FireWorks fireworks = new FireWorks(w,h);
      startTimer();
      while(secTimer(15)) {
        fireworks.render(screen);
        nextFrame();
        sleep(20);
      }
      fadeout();
      fireworks = null;
      System.gc();
    }

    //Slo3d
    {
      Slo3d slo3d = new Slo3d(w, h);
      startTimer();
      while (secTimer(20)) {
        slo3d.render(screen);
        nextFrame();
        sleep(15);
      }

      //Scroller
      Scroller sc = new Scroller(w, h, buffer2);
      startTimer();
      while(secTimer(15)) {
        slo3d.render(buffer1);
        sc.render(buffer1, screen);
        nextFrame();
        sleep(15);
      }
      slo3d = null;
      sc = null;
      fadeout();
      System.gc();
    }
   
    //WhiteNoise
    {
      WhiteNoise wn = new WhiteNoise();
      startTimer();
      while (secTimer(3)) {
        wn.render(screen);
        nextFrame();
        sleep(20);
      }
    }
  }
  

  public static void main(String[] args) {
    int w = 320;
    int h = 200;
    Demo demo = new Demo(w, h);

    boolean bFullscreen = true;
    for (int i = 0; i < args.length; i++) {
      if ("--window".equals(args[i]))
        bFullscreen = false;
      else if ("--longplay".equals(args[i]))
        demo.bLongPlay = true;
    }

    JFrame frame = new JFrame("MegaMix 2005");
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    if (bFullscreen)
      frame.setSize(w, h);
    else
      frame.setSize(w+8, h+27);
    frame.getContentPane().add(demo); 
 
    
    GraphicsEnvironment env = null;
    GraphicsDevice device = null;

    DisplayMode[] modes = null;
    try {
      //vaihdettaan 320x200 fullscreen moodiin, jos sit tuetaan
      if (bFullscreen) {
        env = GraphicsEnvironment.getLocalGraphicsEnvironment();
        device = env.getDefaultScreenDevice();
        modes = device.getDisplayModes();
        for (int i = 0; i < modes.length; i++) {
          if (modes[i].getWidth() == w
              &&modes[i].getHeight() == h
              &&modes[i].getBitDepth() == 32) {
            
            frame.setUndecorated(true);
            device.setFullScreenWindow(frame);
            device.setDisplayMode(modes[i]);
            try {
              Robot robot = new Robot();
              robot.mouseMove(320,200);
            }catch(Exception e) {}
            break;
          }
        }
      }
      
      frame.show();

      demo.requestFocus(); 

      Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
      
      demo.run();

    }
    finally { //pois fullscreenist
      frame.dispose();
      if (bFullscreen)
        device.setFullScreenWindow(null);
    }
    System.exit(0);
  }
}
