
import java.util.Random;

public class Electro {
  private int[] atan;

  private static final int LKM = 8;
  private int[] vx = new int[LKM];
  private int[] vy = new int[LKM];
  private int[] vdx = new int[LKM];
  private int[] vdy = new int[LKM];
  
  private int[] vq = new int[LKM];
  

  private int h,w;
    
  private Random random;
  

  public Electro(int width, int height, int[] buffer) {
    w = width;
    h = height;
    random = new Random(0xBADDAD);

    atan = buffer;
    
    initQs();
    makeTables();

  }
  
  private void initQs() {
    for (int i = 0; i < LKM; i++) {
      do{
        vq[i] = random.nextInt(5)-2;
      }while(vq[i] == 0);

      vx[i] = random.nextInt(w);
      vy[i] = random.nextInt(h);
      vdx[i] = random.nextInt(13)-6;
      vdy[i] = random.nextInt(13)-6;
    }
  }

  private final int colorCircle(double ang) {
    int [] basec = new int[] 
      {0x00ff0000, 0x00ffff00, 0x0000ff00,
       0x0000ffff,0x000000ff,0x00ff00ff};
    double div = 2.0*Math.PI/6.0;  
    double c = ang+Math.PI;
    if (c < 0)
      c = 0;
  
    
    int base = (int)(c / div);
    
    double diff = c/div - base;
    if (base == 6)
      base = 0;
    int bc = basec[base];
    int r = (bc>>16) & 0xff;
    int g = (bc>>8) & 0xff;
    int b = (bc) & 0xff;
    base++;
    if (base >= 6)
      base = 0;
    bc = basec[base];
    int r2 = (bc>>16) & 0xff;
    int g2 = (bc>>8) & 0xff;
    int b2 = (bc) & 0xff;
    
    r += (int)((r2-r)*diff);
    g += (int)((g2-g)*diff);
    b += (int)((b2-b)*diff);        
    
    return 0xff000000|(r<<16)|(g<<8)|b;
  }
  
  private void makeTables() {
    int offs = 0;
    int midx = w >> 1;
    int midy = h >> 1;
    for (int y = 0; y < h; y++) {
      for (int x = 0; x < w; x++) {
        int dx = x-midx;
        int dy = y-midy;
        atan[offs++]   = colorCircle(Math.atan2(dy,dx));
        
      }
    }
  }

  public void render(int [] screen) {
    for (int i = 0; i < LKM; i++) {
      vx[i] += vdx[i];
      if (vx[i] < 0) {
        vx[i] = -vx[i];
        vdx[i] = -vdx[i];
      }
      else if (vx[i] >= w) {
        vx[i] = (w<<1)-vx[i]-1;
        vdx[i] = -vdx[i];
      }
      vy[i] += vdy[i];
      if (vy[i] < 0) {
        vy[i] = -vy[i];
        vdy[i] = -vdy[i];
      }
      else if (vy[i] >= h) {
        vy[i] = (h<<1)-vy[i]-1;
        vdy[i] = -vdy[i];
      }
    }
    
    int max = w*h;
    int offs = 0;
    for (int y = 0; y < h; y++) {
      for (int x = 0; x < w; x++) {
        double ex=0,ey=0;
        for (int i = 0; i < LKM; i++) {
          int dx = x-vx[i];
          int dy = y-vy[i];
          int d = dx*dx+dy*dy;
          double r = Math.sqrt(d);
          double q = vq[i]/(d*r);
         
          ex += q*dx;
          ey += q*dy;
        }
        double ax = ex < 0 ? -ex : ex;
        double ay = ey < 0 ? -ey : ey;
        double dmax = ay > ax ? ay : ax;
        double s = 80/dmax;
        int ix = (w>>1)+(int)(s*ex);
        int iy = (h>>1)+(int)(s*ey);
    
        screen[offs++] =  atan[ix + iy*w];
      }
    }    
  }

}
