import java.awt.image.PixelGrabber;
import java.awt.Image;
import javax.imageio.ImageIO;
import java.io.File;
import java.io.IOException;
public class Flare {
  
  private int[] light;
  private int[] fl;
  private int[] lmask;

  private int w;
  private int h;



  public Flare(int width, int height) {

    
    w = width;
    h = height;
   
    fl = new int[128*128];
  
      light = getImage("rednova.png");
      lmask = getImage("whitenova.png");
  

      
  }
  
  public int[] getImage(String sFile) {
    Image image = null;
    try {
      image = ImageIO.read(getClass().getClassLoader().getResourceAsStream(sFile));
    }
    catch(IOException e) {
      throw new RuntimeException("Unable to find "+sFile+".");
    }
    int w, h;
    w = image.getWidth(null);
    h = image.getHeight(null);
    int[] pixels = new int[w*h];

    
    PixelGrabber pg
      = new PixelGrabber(image,0,0,w,h,pixels,0,w);
    try {
      pg.grabPixels();
    }
    catch (InterruptedException e) {}
    return pixels;
  }  


  
  private final int getLight(int dx, int dy, int[] l) {
    if (dx < -63 || dx > 64 || dy < -63 || dy > 64)
      return 0;
    else
      return l[((64-dy)<<7) + 64-dx];
  }

  public void render(int [] screen) {
    int offs;
    int max = screen.length;
    int time = (int)(System.currentTimeMillis());
    int cosm = (int)(1024*Math.cos( time * Math.PI / 1000 ));
    int sinm = (int)(1024*Math.sin( time * Math.PI / 1000 ));
    int cosl = (int)(1024*Math.cos( -time * Math.PI / 3333 ));
    int sinl = (int)(1024*Math.sin( -time * Math.PI / 3333 ));


    for (int i = 0; i < max; i++) 
      screen[i] = 0xff000000;
   
    for (int y = -63; y < 64; y++)
      for (int x = -63; x < 64; x++) {
        int c  = getLight((cosl*x+sinl*y)>>10, (-sinl*x + cosl*y)>>10, light);
        int m  = getLight((cosm*x+sinm*y)>>10, (-sinm*x + cosm*y)>>10, lmask)&0xff;
                
        int r = (((c >> 16)& 0xff)*m)>>8;
        int g = (((c >>  8)& 0xff)*m)>>8;
        int b = (((c      )& 0xff)*m)>>8;
        if (r > 0xff)
          r = 0xff;
        if (g > 0xff)
          g = 0xff;
        if (b > 0xff)
          b = 0xff;
        fl[((y+63)<<7)+x+63] = 0xff000000 | (r<<16) | (g<<8) | b;
      }
    
    int cx;
    int cy;
      
    int [][] curve = 
      new int[][] {{2,40,0}, {-1,40,0}};

    for (int t = time-1500; t < time; t+=200) {
      float fx=0,fy=0;
      for (int i = 0; i < curve.length; i++) {
        fx += curve[i][1]*Math.cos(curve[i][0]*(t)/(Math.PI*200)
                                   +curve[i][2]*Math.PI/1000.0);
        fy += curve[i][1]*Math.sin(curve[i][0]*(t)/(Math.PI*200)
                                   +curve[i][2]*Math.PI/1000.0);
      }
      
      
      cx = (w>>1)+(int)fx;
      cy = (h>>1)+(int)fy;
      int miny = cy-64;
      int maxy = cy+64;
      if (miny < 0)
        miny = 0;
      if (maxy >= h)
        maxy = h-1;
      int minx = cx-64;
      int maxx = cx+64;
      if (minx < 0)
        minx = 0;
      if (maxx >= w)
        maxx = w-1;
      int oy;
      for (int j = miny; j < maxy; j++) {   
        int cdy = j-cy;
        oy = w*j;
        for (int i = minx; i < maxx ; i++) {
          int dx = 0;
          int dy = 0;
          
          int x = dx+(i-cx);
          int y = dy+(cdy);
          
          int c = getLight(x,y,fl);
         
          int r = ((c >> 16)& 0xff);
          int g = ((c >>  8)& 0xff);
          int b = ((c      )& 0xff);
          
          offs = oy + i ;
          int ct = screen[ offs ];
          r += ((ct >> 16)& 0xff);
          g += ((ct >>  8)& 0xff);
          b += ((ct      )& 0xff);
          
          if (r > 0xff)
            r = 0xff;
          if (g > 0xff)
            g = 0xff;
          if (b > 0xff)
            b = 0xff;
          
          screen[ offs ] = (0xff000000 | (r<<16) | (g<<8) | b);
          
        }
      }
    }
  }
}
