

public class Fractal {
  private int[] colorMap = new int[256];
  private int h,w;
  
  public Fractal(int width, int height) {
    w = width;
    h = height;
        
    initColorMap();
  }

  private void initColorMap() {
    int r = 255;
    int g = 0;
    int b = 0;
    // musta -> punainen
    for (int i = 0; i < 128; i++) {
      colorMap[i] = 0xff000000 | (((int)(( i * r )/128.0))<<16);
    }
    
    int g2 = 255;
    int b2 = 0;
    // punainen -> keltainen
    for (int i = 128; i < 192; i++) {
      int green = g + (int)(( (i-128) * ( g2-g))/64.0);
      int blue = b + (int)(( (i-128) * ( b2-b))/64.0);
      colorMap[i] = 0xff000000 |(255 <<16)|(green<<8)|blue; 
    }

    b = 255;
    // keltainen -> valkoinen
    for (int i = 192; i < 256; i++) {
      int blue = b2 + (int)(( (i-192) * ( b-b2))/64.0);
      colorMap[i] = 0xff000000 |(255 <<16)|(255<<8)|blue; 
    }
  }

  /*
  private static final int julia(int n, double r, double i, double u, double v) {
    for (; n > 0; n--) {
      double r2 = r*r-i*i + u;
      double i2 = 2*r*i   + v;
      r = r2;
      i = i2;
    }

    if (r < -1 || r > 1 || Double.isNaN(r))
      return 0;
    return 0xff-(int)(Math.sqrt(r*r+i*i) * 0xff);
  }
  */

  
  private double s = 2;
  private int recdepth = 18;
  private int counter = 0;
  private final double dx = -0.5350850079702555;
  private final double dy = -0.2385409529157399;    
  
  public boolean render(int [] screen) {
    
    int offs = 0;

    double u = -0.50;
    double v = 0.56;
    
    s *= 0.97;
    
    double ymax = dy+s*((double)(h-h/2))/h;
    double yadd = s/h;
    double xmax = dx+s*((double)(w-w/2))/w;
    double xadd = s/w;
    double y = dy-s/2;

    if (s < 1E-11)
      return false;

    counter ++;
    if (counter > 40) {
      counter = 0;
      recdepth +=2;
    }
    int rechalf = recdepth>>1;      
    
    for (int j=0; j < h; y+=yadd, j++) {
      double x = dx-s/2;
      for (int i=0; i < w; x+=xadd, i++) {
        //screen[offs++] = julia(recdepth, x, y, u, v );
        
        double r = x;
        double img = y;
        double r2;
        double i2;
        int n = recdepth;

        for (; n > rechalf; n-=4) {
          r2  = r*r-img*img + u;
          i2  = 2*r*img     + v;
          r   = r2*r2-i2*i2 + u;
          img = 2*r2*i2     + v;
          r2  = r*r-img*img + u;
          i2  = 2*r*img     + v;
          r   = r2*r2-i2*i2 + u;
          img = 2*r2*i2     + v;
        }
        if (r < -3 || r > 3) {
          screen[offs++] = 0xff000000;
          continue;
        }
        
        for (; n > 0; n-=2) {
          r2 = r*r-img*img + u;
          i2 = 2*r*img     + v;
          r = r2*r2-i2*i2+u;
          img = 2*r2*i2+v;

        }
        if (r < -1 || r > 1 || Double.isNaN(r))
          screen[offs++] = 0xff000000;
        else {
          int value = 0xff-(int)(Math.sqrt(r*r+img*img) * 0xff);
          screen[offs++] = colorMap[value > 0xff ? 0xff : value < 0 ? 0: value];
        }
      }
    }
    return true;
  }
}

