

public class LookingGlass {
  private int[] lookingGlass;  
  private int w, h;
   
  private static final int GLASSES = 8;
  
  private int[] cx = new int[GLASSES];
  private int[] cy = new int[GLASSES];
  private int[] dx = new int[GLASSES];
  private int[] dy = new int[GLASSES];
  

  public  LookingGlass(int width, int height) {

    w = width;
    h = height;
    
    lookingGlass = new int[256*256];
    initLookingGlass();
    
    initGlasses();
  }
  
  private void initGlasses() {
    java.util.Random r = new java.util.Random(0xF00);
    for (int i = 0; i < GLASSES; i++) {
      cx[i] = r.nextInt(w);
      cy[i] = r.nextInt(h);
      dx[i] = r.nextInt(9)-4;
      dy[i] = r.nextInt(9)-4;
    }
  }
  
  private void initLookingGlass() {
    
    for (int x = 0; x < 256; x++)
      for (int y = 0; y < 256; y++) {
	
	int dx = x-128;
	int dy = y-128;
	int D = (int)(2*Math.sqrt(dx*dx + dy*dy));

	if (D > 256)
	  D = 256;
	
	lookingGlass[(y<<8) + x] =  D; 
      }
  }

  public final void render(int [] backgroud, int [] screen) {
    
    for (int i = 0; i < GLASSES; i++) {
      cx[i] += dx[i];
      cy[i] += dy[i];
      if (cx[i] < 0 || cx[i] >= w)
        dx[i] = -dx[i];
      if (cy[i] < 0 || cy[i] >= h)
        dy[i] = -dy[i];
    }

    System.arraycopy(backgroud, 0, screen, 0, screen.length);
    int offs = 0;    
    for (int y = 0; y < h; y++) {
      for (int x = 0; x < w; x++, offs++) {
        
        int zx = 0;
        int zy = 0;
        int ri = 0;
        int gi = 0;
        int bi = 0;
        for (int i = 0; i < GLASSES; i++) {
          int xDiff = (cx[i]-x);
          if (xDiff < -63 || xDiff >= 63)
            continue;
          int yDiff = (cy[i]-y);
          if (yDiff < -63 || yDiff >= 63)
            continue;
          int lgX  = 128+(xDiff<<1); 
          int lgY  = 128+(yDiff<<1);
          
          int zoom = lookingGlass[(lgY<<8)+lgX]; 
          
          zy += ((zoom * yDiff)>>8) - yDiff;
          zx += ((zoom * xDiff)>>8) - xDiff;
        }
        
        int ny  = (y-zy);
        if (ny < 0 || ny >= h)
          continue;
        int nx = x-zx;
        if (nx < 0 || nx >= w)
          continue;
	screen[offs] = backgroud[ny*w+nx]; 
      }
    }
  }
}
