public class Mountains {
  
  private int[] landscape;
  private int[] zBuffer;
  private int[] texture;

  private int h,w,l;
  private double dz = 0;
  private double dx = 0;
  
  public Mountains(int width, int height) {
    w = width;
    h = height;
    l = 256;
    
    landscape=new int[w * l];
    texture = new int[w*l];
    setTexture();
      
    zBuffer = new int[w];
    initLandscape();
  }
  
  private void setTexture() {
    
    int mx = w>>1;
    int my = l>>1;
    int [] miniTex = new int[mx * my];
    Noise noise = new Noise(mx,my);
    noise.render(miniTex);
    
    for (int y = 0; y < my; y++)
      for (int x = 0; x < mx; x++) {
        texture[y*w+x] = miniTex[y*mx+x];
        texture[y*w+w-x-1] = miniTex[y*mx+x];
        texture[(l-y-1)*w+x] = miniTex[y*mx+x];
        texture[(l-y-1)*w+w-x-1] = miniTex[y*mx+x];
      }
  }
  
  private void initLandscape() {
    int H = 200;
 
    for(int y = 0; y < l; y++) {
      double a = 100*Math.cos(y*3*2*Math.PI / l +0.345);
      double b = 14*Math.PI/(734+ 30*Math.cos(y*3*2*Math.PI / l +0.345));
      for (int x = 0; x < w; x++) {
        landscape[y*w+x] = 10+(int)(a*Math.sin(3*y*2*Math.PI /(834+ 80*Math.cos(x*2*Math.PI / l )))
                                    * Math.sin(x * b)
                                    * Math.sin(x*2*Math.PI/w));
      }
    }
  }



  public void render(int [] screen) {
    long time = System.currentTimeMillis();
    dz += 3+Math.abs(4*Math.cos(time*2*Math.PI / 16000));
    dx += 4*Math.sin(time*2*Math.PI / 4667);
    if (dz >= l)
      dz -= l;
    
    if (dx >= w)
      dx -= w;
    else if (dx < 0)
      dx +=w;
    int land = ((int)dz)*w+((int)dx);

    int n= w*h;
    for (int i = 0; i < n; i++)
      screen[i] = 0xff000000; 
    for (int i = 0; i < w; i++)
      zBuffer[i] = 0;

    n= w*l;
    for (int z = 0; z < l; z++) {
      for (int x = 0; x < w; x++, land++) {
        if (land >= n) land -= n;
        //                           z>>2  == perspektiivi
        int diff = landscape[land]+ (z>>2) - zBuffer[x];
        if (diff > 0) {
          int y = h-zBuffer[x]-1;
          zBuffer[x] += diff;
          if (y < diff)
            continue;
          int max = (y-diff)*w+x;
          int I =  (255-z);
          int col = texture[land];
          int r = (((col >> 16) & 0xff)*I)>>8;
          int g = (((col >> 8) & 0xff)*I)>>8;
          int b = (((col) & 0xff)*I)>>8;
          col = 0xff000000 | (r << 16) | (g << 8) | b;
          
          for (y = y*w+x ; y > max; y-=w)
            screen[y] = col;
        }
      }
    }
  }
}
